#!/bin/bash
# Update:       $Id: install.sh,v 1.7 2006/04/10 18:32:34 sbrown Exp $ 
# Purpose:      Install script for CNC10 mill & lathe disks
# Note:         
# Copyright (c) 2006 Centroid, Inc. all rights reserved

# Inputs:       command line parameter 1          = the path to the zip file(s)
#               command line parameter 2          = "--mill", update mill files
#                                                   "--abort-mill", abort a mill update
#                                                   "--mill-no-backups", update mill files without backing up or checking disk usage
#                                                   "--lathe", update lathe files
#                                                   "--abort-lathe", abort a lathe update
#                                                   "--lathe-no-backups", update lathe files without backing up or checking disk usage
#               [mill,lathe]-disk[1,2,3,etc].zip  = the zip file(s) to be extracted
#               Manifest-disk[1,2,3,etc]          = the file manifest within the zip files
#               _next_disk                        = the file containing the next zip number to insert into the drive

# Outputs:      exit status                       = the exit staus of this script
#               _install.out                      = the text to display after this script exits
#               history.txt                       = the history of actions performed by this script (included in a report)

ZIP_PATH="$1"

MILL_ZIP="mill-disk"
LATHE_ZIP="lathe-disk"

DEST_PATH="/cncroot/c"

NEXT_DISK_FILE="/cncroot/c/_next_disk"
OUT_FILE="/cncroot/c/_install.out"
UPDATE_LOG="/cncroot/c/history.txt"

DISPLAY_MESSAGE_EXIT="1"
POWEROFF_CONTROL_EXIT="68"
NEXT_DISK_EXIT="70"

# this awk program takes a "unzip -l" file list and extracts the file and directory paths
AWK_EXTRACT_LIST_FROM_MANIFEST='/ +[0-9]+ +[0-9\-]+ [0-9:]+ +.*/ {print $4}'
AWK_EXTRACT_ONLY_FILES_FROM_MANIFEST='/..-..-.. ..:..   .*[^/]$/ { print $4 }'
AWK_EXTRACT_UNZIPPED_SIZE='/^ +[0-9]+ +[0-9]+ files$/ {print $1}'
AWK_EXTRACT_FREE_DISK_SPACE='/ +[0-9]+% +\/.*$/ {print $4}'

# Description:  restores the backed-up files after a failed update
# $OUT_FILE must exist prior to this function call
function restore_files
{
  if [ -d $BACKUP_PATH ]
  then
    for MANIFEST in `ls Manifest*`
    do
      # do not try to copy directories, only files
      for FILENAME in `awk "$AWK_EXTRACT_ONLY_FILES_FROM_MANIFEST" $MANIFEST`
      do
        cp -dp $BACKUP_PATH/$FILENAME $FILENAME
      done

      rm -f $MANIFEST
    done

    # clean up intermediate files
    rm -f $NEXT_DISK_FILE

    echo "Update aborted, original files restored"

    echo "Update aborted, original files restored"        >> $OUT_FILE

    echo "--- `date` ---"                                 >> $UPDATE_LOG
    echo "Files restored after aborted update"            >> $UPDATE_LOG
  else
    echo "---`date`---"                                   >> $UPDATE_LOG
    echo "Unable to locate backup directory $BACKUP_PATH" >> $UPDATE_LOG
  fi

  sync
}

# Description:  prints the usage of this script and exits
function print_usage
{
  echo "install.sh <zip file path> [--mill,--abort-mill,--mill-no-backups,--lathe,--abort-lathe,--lathe-no-backups,--abort]"
  echo "  --mill, update mill files"
  echo "  --abort-mill, abort a mill update"
  echo "  --mill-no-backups, update mill files without backing up or checking disk usage"
  echo "  --lathe, update lathe files"
  echo "  --abort-lathe, abort a lathe update"
  echo "  --lathe-no-backups, update lathe files without backing up or checking disk usage"
  echo "  --abort, abort all ongoing updates"
  echo "  specifying only the installation path will auto-detect mill/lathe installation with backups"
}

# main script body starts here
cd $DEST_PATH

# remove previous output file if still there
rm -f $OUT_FILE    > /dev/null 2>&1

# check if the zip path was provided
if [ ! -d "$ZIP_PATH" ]
then # valid zip path was not provided, try floppy
  ZIP_PATH="/cncroot/a"
fi

MILL_FILES=("$ZIP_PATH"/${MILL_ZIP}?.zip)
LATHE_FILES=("$ZIP_PATH"/${LATHE_ZIP}?.zip)

# check if install script was explicitly called as a mill or lathe update
case "$2" in
  "--mill" | "--abort-mill" | "--mill-no-backups" )
    # script called as a mill update
    LATHE_FILES=()
    ;;

  "--lathe" | "--abort-lathe" | "--lathe-no-backups" )
    # script called as a lathe update
    MILL_FILES=()
    ;;

  "--abort" | *)
    ;;
esac

# check which zip file is on update disk
if [ "$MILL_FILES" ]
then  # this is a mill update
  CNC10_PATH="cnc10"
  BACKUP_PATH="cnc10bak"
  ZIP_FILE=$MILL_ZIP
  UPDATE_TYPE="Mill"
  CNC10_EXE="cnc10m.exe"
  # the loop is required to preserve the subscript->filename relationship
  for (( zipindex = 0 ; zipindex < ${#MILL_FILES[*]} ; zipindex++ ))
  do
    ZIP_FILES[$zipindex]=${MILL_FILES[$zipindex]}
  done
elif [ "$LATHE_FILES" ]
then  # this is a lathe update
  CNC10_PATH="cnc10t"
  BACKUP_PATH="cnc10tbak"
  ZIP_FILE=$LATHE_ZIP
  UPDATE_TYPE="Lathe"
  CNC10_EXE="cnc10t.exe"
  # the loop is required to preserve the subscript->filename relationship
  for (( zipindex = 0 ; zipindex < ${#LATHE_FILES[*]} ; zipindex++ ))
  do
    ZIP_FILES[$zipindex]=${LATHE_FILES[$zipindex]}
  done
else
  echo "Error: Not a valid update disk."     > $OUT_FILE

  exit $DISPLAY_MESSAGE_EXIT
fi

# remove previous output file if still there 
rm -f $OUT_FILE    > /dev/null 2>&1

# extract CNC Linux version number and check for min requirements
MIN_VERSION_OK=`awk '{version_string = gensub("v","",1,$3); if (version_string >= 1.25) print version_string}' /etc/issue`

if [ ! "$MIN_VERSION_OK" ]
then
  echo "Current CNC Linux version is:"       >  $OUT_FILE
  cat  /etc/issue                            >> $OUT_FILE
  echo "This update requires CNC Linux v1.25">> $OUT_FILE

  exit $DISPLAY_MESSAGE_EXIT
fi

# now that the environment is set up, check if the script must restore the backup files or omit them entirely
case "$2" in
  "--abort-mill" | "--abort-lathe" | "--abort")
    # script called to abort an update
    restore_files

    # Inform cnc10 that control needs to be powered off.
    exit $POWEROFF_CONTROL_EXIT
    ;;

  "--mill-no-backups" | "--lathe-no-backups" )
    OMIT_BACKUPS="true"
    ;;

  "--mill" | "--lathe" | "" )
    OMIT_BACKUPS=""
    ;;
    
  *)
    print_usage

    exit $DISPLAY_MESSAGE_EXIT
    ;;
esac

###################################################################################################################################
#  # backup system files
#  cp -f $CNC10_PATH/cnc10*.*  $BACKUP_PATH > /dev/null 2>&1
#  cp -f $CNC10_PATH/pc.plc    $BACKUP_PATH > /dev/null 2>&1
#  cp -f $CNC10_PATH/*.src     $BACKUP_PATH > /dev/null 2>&1
#  cp -f $CNC10_PATH/*.hex     $BACKUP_PATH > /dev/null 2>&1
###################################################################################################################################

# prepare to backup files
if [ ! $OMIT_BACKUPS ] && [ ! -d $BACKUP_PATH ]
then
  mkdir $BACKUP_PATH
fi

# unzip cnc10
echo "Installing CNC10 $UPDATE_TYPE update..."

# the NEXT_DISK_FILE does not exist, assume "1"
if [ ! -e $NEXT_DISK_FILE ]
then
  echo "1" > $NEXT_DISK_FILE
fi

for (( zipindex = 0 ; zipindex < ${#ZIP_FILES[*]} ; zipindex++ ))
do
  zipfile="${ZIP_FILES[$zipindex]}"
  if [ "$zipfile" == "$ZIP_PATH/$ZIP_FILE`cat $NEXT_DISK_FILE`.zip" ]
  then
    MANIFEST_FILE="Manifest`echo $zipfile | sed -e 's/.*\(-disk[0-9]\).zip/\1/'`"

    # verify zip integrity and manifest existance
    if unzip -tqq "$zipfile" && unzip -tqq "$zipfile" $MANIFEST_FILE
    then
      if [ ! "$OMIT_BACKUPS" ]
      then
        # calculate the needed size + 20% (for filesystem overhead)
        NEEDED_SPACE="$((`unzip -l "$zipfile" | awk "$AWK_EXTRACT_UNZIPPED_SIZE"`/1000*120/100))"
      else
        NEEDED_SPACE="0"
      fi
      AVAILABLE_SPACE=`df $DEST_PATH | awk "${AWK_EXTRACT_FREE_DISK_SPACE}"`
      if (($NEEDED_SPACE < $AVAILABLE_SPACE))
      then
        # extract the manifest now that we know it is healthy
        unzip -oq "$zipfile" $MANIFEST_FILE

        # verify that the contents of the zip file correspond with the manifest
        FILES_WERE_MISSING=""
        for ZIPPED_FILE in `awk "$AWK_EXTRACT_LIST_FROM_MANIFEST" $MANIFEST_FILE`
        do
          if ! unzip -tqq "$zipfile" $ZIPPED_FILE
          then
            FILES_WERE_MISSING="$FILES_WERE_MISSING $ZIPPED_FILE"
          fi
        done

        if [ ! "$FILES_WERE_MISSING" ]
        then
          # create directories for the new files and backups
          for ZIPPED_FILE in `awk "$AWK_EXTRACT_LIST_FROM_MANIFEST" $MANIFEST_FILE`
          do
            # make the base directory, if it does not already exist, make the backup directory as well
            BASE_DIRECTORY="`echo $ZIPPED_FILE | sed -e 's/^\(.*\/\)*[^/]*$/\1/'`"
            if [ ! -d $BASE_DIRECTORY ]
            then
              mkdir $BASE_DIRECTORY
            fi
            if [ ! "$OMIT_BACKUPS" ]
            then
              if [ ! -d $BACKUP_PATH/$BASE_DIRECTORY ]
              then
                mkdir $BACKUP_PATH/$BASE_DIRECTORY
              fi
            fi
          done

          # backup files prior to unzip
          if [ ! "$OMIT_BACKUPS" ]
          then
            for ZIPPED_FILE in `awk "$AWK_EXTRACT_LIST_FROM_MANIFEST" $MANIFEST_FILE`
            do
              FILENAME="`echo $ZIPPED_FILE | sed -e 's/^.*\/\(.*\)$/\1/'`"
              # do not try to copy directories, only files
              if [ "$FILENAME" ]
              then
                cp -dp $ZIPPED_FILE $BACKUP_PATH/$ZIPPED_FILE > /dev/null 2>&1
              fi
            done
          fi

          # update the system from the zip file (finally)
          unzip -oq "$zipfile"
          sync

          echo "--- `date` ---"                               >> $UPDATE_LOG
          echo "CNC10 $UPDATE_TYPE disk $zipfile installed."  >> $UPDATE_LOG
        else
          echo "--- `date` ---"                                                 >> $UPDATE_LOG
          echo "Missing files from update file ${zipfile}: $FILES_WERE_MISSING" >> $UPDATE_LOG

          echo "Missing files from update file ${zipfile}:" >  $OUT_FILE
          echo "$FILES_WERE_MISSING"                        >> $OUT_FILE

          restore_files

          # Inform cnc10 that control needs to be powered off.
          exit $POWEROFF_CONTROL_EXIT
        fi
      else
        echo "---`date`---"                             >> $UPDATE_LOG
        echo "Insufficient disk space to update system" >> $UPDATE_LOG

        # though NEEDED_SPACE could be 0, AVAILABLE_SPACE would need to be negative to get here
        echo "Insufficient disk space to update system"                     >  $OUT_FILE
        echo "Needed: ${NEEDED_SPACE}000, Available: ${AVAILABLE_SPACE}000" >> $OUT_FILE

        restore_files

        # Inform cnc10 that control needs to be powered off.
        exit $POWEROFF_CONTROL_EXIT
      fi
    else
      echo "--- `date` ---"                       >> $UPDATE_LOG
      echo "Disk $zipfile failed integrity check" >> $UPDATE_LOG

      echo "Disk $zipfile failed integrity check" >  $OUT_FILE

      restore_files

      # Inform cnc10 that control needs to be powered off.
      exit $POWEROFF_CONTROL_EXIT
    fi
  fi
done

# the update is either finished, or needs to continue... clean up or prompt for the next disk
if [ "`cat $NEXT_DISK_FILE`" == "done" ]
then
  # done with the update, delete temporary files
  rm -f $NEXT_DISK_FILE Manifest*

  # remove cnc9.hex if it was not there to begin with
  if [ -d "$BACKUP_PATH/$CNC10_PATH" -a ! -r "$BACKUP_PATH/$CNC10_PATH/cnc9.hex" ]
  then
      rm -f $CNC10_PATH/cnc9.hex
  fi

  # remove obsolete files
  rm -f "$CNC10_PATH/ncfiles/CONVGRAF.\$NC"  > /dev/null  2>&1

  # output new cnc10 version to update log
  cd $DEST_PATH/$CNC10_PATH
  echo "--- `date` ---"                                   >> $UPDATE_LOG
  echo "Completed Update to " `bin/$CNC10_EXE --version`  >> $UPDATE_LOG
  sync

  # Inform cnc10 that control needs to be powered off.
  exit $POWEROFF_CONTROL_EXIT
else
  echo $UPDATE_MESSAGE                                                  >  $OUT_FILE
  echo "Please install CNC10 $UPDATE_TYPE disk `cat $NEXT_DISK_FILE`."  >> $OUT_FILE

  exit $NEXT_DISK_EXIT
fi
